#!/usr/bin/python
# -*- coding: utf-8 -*-
# checksysreport, a tool to spot unsupported issues.
# Copyright 2005 Pierre Amadio 
# pamadio@redhat.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
import sys
sys.path.append('/usr/share/checksysreport')
import rhnapiwrap
import globals
import xmlrpclib
import os
import re
import string
import syslog
import copy

class sysreportChecker:
    "main class for sysreport checker"
    #a rhn connector.
    rhn=rhnapiwrap.rhnapiwrap('','')
    #information related to up2date rpms according to a given channel
    #(the dictionnary key)
    latest_packages_cache={}
    #information related to all rpms according to a given channel
    all_packages_cache={}
    #all existing sofwtare channels
    all_software_channels=[]
    #files in a rpm according to the package_id (This will be unused now that the api works with  packages.list_files )
    files_in_rpm_cache={}
    #information related to files in a given package
    filesInfo_in_rpm_cache={}
    #changelog associated with the package package_id
    changelog_cache={}
    #erratas per packages.
    erratas_cache={}
    #bugs fixed by a given errata cache
    bugs_per_errata_cache={}
    osRelease='unknown'
    #as es ws desktop
    osFlavour='unkown'
    osArch='unknown'
    sysreportPath=''
    baseChannel='unknown'
    childsChannels=[]
    #supportedLatestPackages:
    #A list of dictionnary of packages that are supposed to be usable and
    #up2date for all the channels the system can be subscribed to.
    #The dict is structured as the output of
    #https://rhn.redhat.com/rpc/api/channel/software/list_latest_packages/
    #In order to be able to check from wich channel the rpm is supposed to
    #come from, let's add a channel_label key. 
    supportedLatestPackages=[]
    #installedRpms: a list of dictionnary based on the content of
    # sysreport's installed-rpms
    # keys are: package_name,package_version,package_release,package_arch_label
    installedRpms=[]

    def __init__(self,sysreportPath='',action=0,options=[]):
        """
        Constructor. sysreportPath is the path to the directory where the
        sysreport is untarred
        action: 
                equal 2 -> we want to compare some changelogs
        """
        login=globals.config['rhn_login']
        password=globals.config['rhn_password']
        self.options=options
        globals.rhnConnector.login()
        self.rhn=rhnapiwrap.rhnapiwrap(login,password,globals.config['rhn_server'])
        self.rhn.login()
        self.load_channels_info()
        if action==2:
            if len(self.options.pidToDiff)!=2:
                globals.die("You need two set package id with -d if you want a comparison to be made")
            pid1=self.options.pidToDiff[0]
            pid2=self.options.pidToDiff[1]
            rpm1=self.get_package_by_pid(pid1)
            if not rpm1:
                globals.die("Unable to find package for id="+pid1)
            rpm2=self.get_package_by_pid(pid2)
            if not rpm2:
                globals.die("Unable to find package for id="+pid2)

            for cur in [rpm1,rpm2]:
                name=cur['package_name']+"-"+str(cur['package_version'])+"-"\
                      +str(cur['package_release'])+\
                      " ("+cur['package_arch_label']+") in "\
                      +cur['channel_label']
                print str(cur['package_id'])+" -> "+name

            diff=self.diff_changelog(rpm1,rpm2)
            dates=diff.keys()
            dates.sort()
            changelogData=0
            for d in dates:
                changelogData=1
                print diff[d]['date']
                print diff[d]['header']
                print diff[d]['changes']
            if not changelogData:
                #There was no data in changelog diff, a bit of debug is needed:
                print "No data in the diff...."
                print "rpm1=",rpm1
                print "rpm2=",rpm2

            print " "
            globals.die("")
        
        self.sysreportPath=sysreportPath
        self.osFlavour=self.detect_osFlavour().lower()
        self.osArch=self.detect_osArch()
        self.installedRpms=self.getInstalledRpms()
        self.baseChannel=self.detect_baseChannel()
        self.childsChannels=self.list_all_childs_channels(self.baseChannel)
        #TODO: if arch is 686 add all i386 subchannels to self.childsChannels
        #as well .
        #Hmmm, and maybe removing the beta channels as well....
        for p in self.get_latest_rpm_in_channel(self.baseChannel):
            p['channel_label']=self.baseChannel
            self.supportedLatestPackages.append(p)
        for c in self.childsChannels:
            reg=re.compile(r'beta')
            betaSearch=reg.search(c)
            #We don't want to support beta channels.
            if betaSearch==None:
                for p in self.get_latest_rpm_in_channel(c):
                    p['channel_label']=c
                    self.supportedLatestPackages.append(p)


    def get_existing_advisories(self):
        """
         return a list of all known advisories for channel we support.
        """
        out=[]
        for c in self.all_software_channels:
            for adv in self.rhn.listErrata(c['channel_label']):
                advName=adv['errata_advisory']
                if advName not in out:
                    out.append(advName)
        return out



        
    def load_channels_info(self):
        """
        Load pickled data from path directory
        """
        self.all_software_channels=globals.select_allChannelsInfo()


    def get_file(self,file):
        """
        return the content of a file relative to the sysreport's root
        directory
        """
        fullpath=self.sysreportPath+"/"+file
        if(os.path.isfile(fullpath)):
            f=open(fullpath,'r')
            content=f.read()
            f.close()
            return content
        else:
            #globals.die("No such file to open:"+fullpath)
            raise("No such file to open:"+fullpath)

    def get_kernel_version(self):
       """
        Return the kernel version used when the sysreport was launched.
        such as : 2.6.9-5.0.5.EL  2.4.21-27.0.4.ELsmp and so on.
       """
       uname=self.get_file('uname')
       reg=re.compile(r"^Linux (\S+) (\S+) #",re.MULTILINE)
       if reg.search(uname):
           return reg.search(uname).group(2)
       else:
           globals.die("Cannot detect kernel version with:"+uname)
       

    def list_kernel_related_rpms(self):
        """
         Return a list of rpm that ship a kernel that match the current
         system 's arch and group of available channel.
         TODO: GFS module are not returned.
        """
        out=[]
        channels=[self.baseChannel]
        for c in self.childsChannels:
            if not re.compile('beta').search(c):
                #We do not want to know about kernel coming from a beta channel.
                channels.append(c)

        for c in channels:
            rpms=self.get_all_rpm_in_channel(c)
            for p in rpms:
                pname=p['package_name']
                p['channel_label']=c
                regKer=re.compile("kernel-?(.*)")
                if regKer.search(pname) \
                       and not re.compile("devel").search(pname) \
                       and not re.compile("-doc").search(pname) \
                       and not re.compile("-unsupported").search(pname) \
                       and not re.compile("-utils").search(pname):
                    out.append(p)
        return out

    
    def get_latest_rpm_in_channel(self,channel_label):
        """
        return a list of up2date rpms info in a given channel
        """
        if self.latest_packages_cache.has_key(channel_label):
            return self.latest_packages_cache[channel_label]
        #We got to get data from disk.
        #fileName=globals.channelPackageFilename2(channel_label,"latest")
        #tmp=globals.getData(fileName)
        tmp=globals.select_all_package_in_channel(channel_label,True)
        self.latest_packages_cache[channel_label]=tmp
        return tmp

    def get_erratas_fixed_by_rpm(self,pid):
        """
         Return a list (it looks like the size is always 1 ot 0 though)
         of RHGA solved by a given rpm.
        """
        if self.erratas_cache.has_key(pid):
            return self.erratas_cache[pid]
        else:
            """
            fileName=globals.errataFilename(str(pid))
            if globals.existFile(fileName):
               tmp=globals.getData(fileName)
            else:
               tmp=[]
            """
            tmp=globals.get_erratas_fixed_by_rpm(pid)
            self.erratas_cache[pid]=tmp
        return tmp

    def list_new_package_for(self,ancestor):
        """
         Return a list of up2dates packages that would apply for
         an update of ancestor.
        """
        out=[]
        for candidate in self.get_all_rpm_in_channel(ancestor['channel_label']):
            if candidate['package_name']==ancestor['package_name'] and\
                   candidate['package_arch_label']==ancestor['package_arch_label']:
                if self.rpm_is_younger_than(candidate,ancestor):
                    out.append(candidate)
                                                              
        return out

    def rpm_is_younger_than(self,candidate,ancestor):
        """
         Return true if candidate is younger than ancestor
        """
        if candidate['package_name']!=ancestor['package_name']:
            globals.die('you dont want to compare apples with pears...')
        #if candidate['package_id']>ancestor['package_id']:
        if globals.rpm_labelCompare(candidate,ancestor)>0:
            return True
        else:
            return False


    def list_advisories(self,ancestor):
        """
        Return a list of advisories not being fixed by
        the package ancestor
        """
        out=[]
            
        ancestor_adv=self.get_erratas_fixed_by_rpm(ancestor['package_id'])
        for child in self.list_new_package_for(ancestor):
            #print "child",child
            for rhea in self.get_erratas_fixed_by_rpm(child['package_id']):
                if rhea not in ancestor_adv and rhea not in out:
                    """
                    Just in case, cf RHSA-2008:0030 for pid 430732
                    and 431141
                    """
                    out.append(rhea)
        return out        
    
        
    def get_all_rpm_in_channel(self,channel_label):
        """
        return a list of all rpms info in a given channel
        """
        if self.all_packages_cache.has_key(channel_label):
            return self.all_packages_cache[channel_label]
        tmp=globals.select_all_package_in_channel(channel_label)
        self.all_packages_cache[channel_label]=tmp
        return tmp


    def get_bugs_for_errata(self,advisory):
        """
         Return information related to bugs solved by a given advisory.
        """
        if self.bugs_per_errata_cache.has_key(advisory):
            return self.bugs_per_errata_cache[advisory]
        else:
            tmp=globals.get_bugs_for_errata(advisory)
            self.bugs_per_errata_cache[advisory]=tmp
            return tmp



    def list_files_in_rpm(self,pid):
        """
        return a list of files shipped with the rpm of id pid
        """


        """
        if self.files_in_rpm_cache.has_key(pid):
            return self.files_in_rpm_cache[pid]
        #May be the data is cached on disk.
        fileName="files-in-"+str(pid)+".pickle"
        if(os.path.isfile(cacheDirectory+"/"+fileName)):
           tmp=globals.getData(fileName)
           self.files_in_rpm_cache[pid]=tmp
           return tmp
        #The data is not cached on disk, let's fetch it
        rpms=self.rhn.list_files_in_package(pid)
        globals.storeData(fileName,rpms)
        self.files_in_rpm_cache[pid]=rpms
        return rpms
        """

        info=globals.list_filesInfo_in_package(pid)
        out=[]
        for i in info:
            out.append(i['file_path'])
        return out



        

    def get_changelog(self,pid):
        """
         Return a dictionnary describing the changelog associated with
         the package having pid for package_id.
        """
        if self.changelog_cache.has_key(pid):
            return self.changelog_cache[pid]
        #maybe the data is cached on disk.
        changelog=globals.get_changelog(pid)
        self.changelog_cache[pid]=changelog
        return changelog
        


    def diff_changelog(self,rpma,rpmb):
        """
          Return the a list of changelog entry appearing only in
          the more recent package between rpma and rpmb
          """
        out={}
        aPid=rpma['package_id']
        bPid=rpmb['package_id']
        
        if rpma['package_name']!=rpmb['package_name'] or \
           rpma['channel_label'].replace('-fastrack','')!=rpmb['channel_label'].replace('-fastrack',''):

            out[101]={'date':'0000-01-01','header':'','changes':'Those package do not come from the same channel: no changelog diff'}
            return out

        return globals.diff_changelog(aPid,bPid)
        


    def list_all_childs_channels(self,parent_label):
        """
        return a list of all childs channels of parent_label
        """
        out=[]
        for c in self.all_software_channels:
            
            if c['channel_parent_label']==parent_label:
                out.append(c['channel_label'])
                childChilds=self.list_all_childs_channels(c['channel_label'])
                if(len(childChilds)):
                   out.append(childChilds)
        return out                   


    def list_base_channels(self):
        """
        return a list of channels label that have no parents
        """
        output=[]
        for c in self.all_software_channels:
            if c['channel_parent_label']==None:
                output.append(c['channel_label'])
        return output

    def list_subscribed_base_channel(self, systemid):
	"""
	return the label of the subscribed base channel
	"""
	output = "unknown"
        try:
            for c in self.rhn.list_base_channels(systemid):
                    if c['CURRENT_BASE']==1:
			return c['LABEL']
        except xmlrpclib.Fault,fault:
            if fault.faultCode==-208:
                '''no such systems'''
                return output
        
	return output

    def get_list_3rdParty_installed_rpms(self):
        out=[]
        for candidate in self.installedRpms:
            pName=candidate['package_name']
            testExist=False
            for supported in self.supportedLatestPackages:
                if supported['package_name']==pName:
                    testExist=True
            if testExist==False:
                out.append(pName)
        return out        

    def guess_package_id(self,target):
        """
         Try to find the package_id that match for 'target' as its
         package_name, package_version, package_release and
         the channel_label.
        """
        out=0
        for curRpm in self.get_all_rpm_in_channel(target['channel_label']):
            
            if curRpm['package_name']==target['package_name'] and \
            str(curRpm['package_version'])==str(target['package_version']) \
            and \
            str(curRpm['package_release'])==str(target['package_release']):
                #May be there would still be a test for arch to perform
                #because of the athlon , i386 and i686 version of rpm ?
                out=curRpm['package_id']
        return out

    def get_obsolete_rpms(self):
        """
        This return a list of dictionnary containing information about the
        current installed package ('current') and the existing up2date one
        ('latest').
        """
        out=[]
        for candidate in self.installedRpms:
            cName=candidate['package_name']

            cRelease=candidate['package_release']
            cVersion=candidate['package_version']
            cArch=candidate['package_arch_label']
            for l in self.supportedLatestPackages:
                lName=l['package_name']
                lRelease=l['package_release']
                lVersion=l['package_version']
                lArch=l['package_arch_label']
                if lName==cName:
                    #bugzilla 152276 is problematic here.
                    if( (str(cRelease)!=str(lRelease) \
                         or str(cVersion)!=str(lVersion))
                       and ( \
                        cArch==lArch\
                        or (cArch=='i386' and lArch=='athlon' )\
                        or (cArch=='i386' and lArch=='i686' ) \
                        or (cArch=='ppc' and re.compile(r"^ppc").search(lArch))
                        )
                       ):
                        
                        newChannel=l['channel_label'].replace('-fastrack','')
                        #candidate['channel_label']=l['channel_label']
                        candidate['channel_label']=newChannel
                        candidate['package_id']=self.guess_package_id(candidate)
                        if(candidate['package_id']!=0):
                            snt="select package_epoch from packages where package_id=%i"%candidate['package_id']
                            globals.db_cursor.execute(snt)
                            epoch=globals.db_cursor.fetchone()[0]
                            if not epoch or re.compile("^\s+$").search(epoch):
                                epoch='None'
                            candidate['package_epoch']=epoch
                        else:
                            candidate['package_epoch']='None'
                        
                        tmpDic={'current':copy.deepcopy(candidate),'latest':l}
                        out.append(tmpDic)

        return out

    def getRunningKernel(self):
        """
        Return a dictionnary describing the package that furnsish
        the current running kernel
        return 0 if detection failed.
        """
        runningKernel=self.get_kernel_version()
        for p in self.list_kernel_related_rpms():
            candidate=p['package_version']+'-'+str(p['package_release'])
            if re.compile("kernel-(.*)").search(p['package_name']):
                candidate=candidate+re.compile("kernel-(.*)")\
                           .search(p['package_name']).group(1)

            if runningKernel==candidate:
                return p
        return 0     


    def kernel_is_tainted(self):
        """
        Return 0 if the kernel is not tainted,
        return the reason it s tainted otherwise
        """
        lsmod=self.get_file('lsmod')
        pattern="^Module\s+Size\s+Used by(.*)$"
        reg=re.compile(pattern,re.MULTILINE)
        if reg.search(lsmod):
            taintedSnt=reg.search(lsmod).group(1)
            if taintedSnt=='' \
                   or re.compile("\s+Not tainted").search(taintedSnt):
                return 0
            else:
                return string.replace(taintedSnt,' ','')
        return "Not able to detect if kernel is tainted:\n%s"%lsmod

    def get_loaded_modules(self):
        """return a list of currently loaded kernel module"""
        lsmod=self.get_file('lsmod')
        out=[]
        pattern=r"^(\S+)\s+\d+"
        reg=re.compile(pattern,re.MULTILINE)
        for m in reg.findall(lsmod):
            if m!='Module':
                out.append(m)
        return out


    def get_supported_kernel_modules(self):
        """
        Return a list of kernel module name that are known to be shipped
        for the current kernel.
        """
        out=[]
        running=self.getRunningKernel()
        if not running :
            globals.die("Unable to detect running kernel, so cannot get a list of supported modules.")
        rpmFiles=self.list_files_in_rpm(running['package_id'])
        for f in rpmFiles:
            pattern=r"^/lib/modules/(.*)/(\S+)\.k?o$"
            if re.compile(pattern).search(f):
                out.append(re.compile(pattern).search(f).group(2))
        #TODO: add potential gfs modules here and eventual child
        #channel's kernel modules.        
        return out
    
    def get_unsupported_kernel_modules(self):
        """Return a list of unsupported loaded modules"""
        out=[]
        loaded=self.get_loaded_modules()
        supported=self.get_supported_kernel_modules()
  	 
        for l in loaded:
            if l not in supported:
                out.append(l)
        return out

    def get_kernel_modules_verify(self):
        """Return a list of entry in rpm-Va that are related to the running kernel"""
        out=[]
        version=self.get_kernel_version()
        dirname="/lib/modules/"+version
        verify=self.get_file('rpm-Va')
        pattern=r"^(.*"+dirname+"+.*)$"
        reg=re.compile(pattern,re.MULTILINE)
        for m in reg.findall(verify):
            out.append(m)
        return out    
        

    def module_is_loaded(self,modName):
        """
        Return True if the module ModName appears loaded in the output of lsmod
        modName is a module name based on the file name without the .o or .ko extention.
        Return False if it does not appearas loaded
        """
        modNameSubst=string.replace(modName,'-','_')
        loaded=self.get_loaded_modules()
        for l in loaded:
            testName=string.replace(l,'-','_')
            if testName==modNameSubst:
                return True
        return False
    


    def get_package_by_pid(self,pid):
        """
         browse all existing rpm in all channels and returns the information
         related to the rpm with id pid.
        """
        pid=int(pid)
        for channel in self.all_software_channels:
            for package in self.get_all_rpm_in_channel(channel['channel_label']):
                if pid==int(package['package_id']):
                    package['channel_label']=channel['channel_label']
                    return package
        return 0        


    def get_up2date_conf(self):
        """
         Return a dictionnary of value set in /etc/sysconfig/rhn/up2date
         Return 0 if there was no /etc/sysconfig/rhn/up2date file.
        """
        out={}
        fileName="etc/sysconfig/rhn/up2date"
        if not os.path.isfile(self.sysreportPath+"/"+fileName):
            return 0
        content=self.get_file(fileName)
        pattern=r"^([^#].*)=(.*)$"
        reg=re.compile(pattern,re.MULTILINE)
        for m in reg.findall(content):
            out[m[0]]=m[1]
        return out    

    def guess_existing_z_streams(self):
        baseName='rhel-'+self.osArch+'-'+self.osFlavour+'-'+self.osRelease
        regBase=re.compile(baseName)
        regZ=re.compile('.*z$')
        out=[]
        for cand in self.list_base_channels():
            if regBase.match(cand) and regZ.match(cand):
                out.append(cand)
        return out


    def get_cpu_infos(self):
        """
        Return a dictionnary describing the cpus according
        to /proc/cpuinfo.
        """
        out={}
        fh=open(self.sysreportPath+"/proc/cpuinfo")
        line_reg=re.compile("^\s*(.*?)\s*:\s*(.*)\s*$")
        cur_cpu_id=-1

        for curline in fh:
            if re.compile("^\s+$").search(curline):
                continue
            
            if line_reg.search(curline):
                cur_key=line_reg.search(curline).group(1)
                cur_value=line_reg.search(curline).group(2)
            else:
                syslog.syslog(syslog.LOG_DEBUG,"(get_cpu_infos) Problem parsing: '%s'"%curline)
                continue

            if cur_key=="processor":
                cur_cpu_id=int(cur_value)
                out[cur_cpu_id]={}
                continue

            if cur_cpu_id<0:
                syslog.syslog(syslog.LOG_DEBUG, "(get_cpu_infos) what happen with : '%s' ?"%curline)
                continue

            out[cur_cpu_id][cur_key]=cur_value
            
        fh.close()
        return out


    def get_cpu_vendor(self):
        """
         Return the first occurence of 'vendor_id' in /proc/cpuinfo
        """
        fileName='proc/cpuinfo'
        if not os.path.isfile(self.sysreportPath+"/"+fileName):
            return "Unknown"
        cpuinfo=self.get_file(fileName)
        pattern="vendor_id\s+:(.*)"
        reg=re.compile(pattern,re.MULTILINE)
        if reg.search(cpuinfo):
            return reg.search(cpuinfo).group(1)
        return "Unkown"


    def get_physical_address_size(self):
        """
        Return the value of physical address size according to
        /proc/cpuinfo:
        address sizes   : 48 bits physical, 48 bits virtual
        """
        cpuinfo=self.get_file('proc/cpuinfo')
        pattern="address sizes\s+:\s+(\d+) bits physical"
        reg=re.compile(pattern,re.MULTILINE)
        if reg.search(cpuinfo):
            return int(reg.search(cpuinfo).group(1))
        return None

    def is_amd(self):
        cpu_vendor=self.get_cpu_vendor()
        if re.compile("AMD").search(cpu_vendor):
            return True
        else:
            return False

    def get_dmi_system_information(self):
        """
        Return a hash describing what the dmidecode information
        about the 'System Information'

        What we got to detect is:

        Handle 0x0100, DMI type 1, 27 bytes.
        System Information
                Manufacturer: Dell Inc.
                Product Name: PowerEdge R900
                Version: Not Specified
                Serial Number: HTVSQ3J
        
        """
        out={}
        dmiFile=self.sysreportPath+"/dmidecode"
        inSystemInfoFlag=False
        if os.path.isfile(dmiFile):
            f=open(dmiFile)
            for curline in f:
                if re.compile("^\s*System Information").search(curline):
                    inSystemInfoFlag=True
                    continue

                if not inSystemInfoFlag:
                    continue

                reg=re.compile("^\s+(.*):\s*(.*)\s+$")
                if reg.search(curline):
                    out[reg.search(curline).group(1)]=reg.search(curline).group(2)
                else:
                    break
            f.close()
            return out
        else:
            return False
        


                                 
    def detect_osFlavour(self):
        "as es ws desktop"
        globals.die("detect_osFlavour has to be done in a subclass")
        
    def detect_osArch(self):
        "i386 ia64 x86_64 ppc s390 s390x"
        globals.die("detect_osArch has to be done in a subclass")

    def detect_baseChannel(self):
        globals.die("detect baseChannel has to be done in a subclass")

    def getInstalledRpms(self):
        globals.die("getInstalledRpms has to be perfomed in a subclass")

    def list_known_kernel_issues(self):
        globals.die("list_known_kernel_issues has to be performed in a subclass")

    def list_known_amd_issues(self):
        globals.die("list_known_amd_issues has to be performed in a subclass")
        
#####################################################################
#
#                            AS 2.1 CLASS
#
#####################################################################

class sysreportChecker_2(sysreportChecker):
    """sysreportChecker for AS2.1 based systems"""

    def __init__(self,sysreportPath='',action=0,options=[]):
        self.osRelease='2.1'
        sysreportChecker.__init__(self,sysreportPath,options=options)


    def detect_osFlavour(self):
        redhatReleasePath=self.sysreportPath+'/etc/redhat-release'
        if os.path.isfile(redhatReleasePath):
            RELEASE=open(redhatReleasePath,'r')
            out='0'
            for line in RELEASE:
                rhelreg2AS=re.compile(r"Red Hat Linux Advanced Server release 2.1.*")
                rhelmatch2AS=rhelreg2AS.match(line)
                rhelreg2ES=re.compile(r"Red Hat Enterprise Linux ES release 2.1.*")
                rhelmatch2ES=rhelreg2ES.match(line)
                if rhelmatch2AS:
                    return 'as'
                if rhelmatch2ES:
                    return 'es'
                else:
                    globals.die("Cannot detect osFlavour for a RHEL2.1, are you the lucky winner of a 'fix aw channel ?' problem")
        else:
            globals.die("Cannot find /etc/redhat-release")
            return out
        
    def detect_osArch(self):
        """
        For AS2.1 arch has to be specified with the -a option
        """
        if not self.options.arch:
            globals.die("Option --arch is mandatory for RHEL2 (i386 or ia64)")
        arch=self.options.arch
        if arch != 'ia64' and arch !='i386':
            globals.die("Only ia64 and i386 architecture are supported for 2.1")
        return arch    
        

    def getInstalledRpms(self):
        out=[]
        #a flag to test if a sysreport from rhel3 is being used on AS2.1
        v3forV2Flag=0

        if os.path.isfile(self.sysreportPath+"/rpm"):
            rpmFile=self.sysreportPath+"/rpm"
        elif os.path.isfile(self.sysreportPath+"/installed-rpms"):
            rpmFile=self.sysreportPath+"/installed-rpms"
        else:
            globals.die("Cannot detect installed-rpms for RHEL2")
            
        
        RPMS=open(rpmFile,'r')
        for r in RPMS:
            p=re.compile(r"^(.*)-(.*)-(.*)$")
            m=p.match(r)
            if m:
                tmpDic={}
                tmpDic['package_name']=m.group(1)
                tmpDic['package_version']=m.group(2)
                tmpDic['package_release']=m.group(3)
                tmpDic['package_arch_label']=self.osArch
                if m.group(3)=='i386' or m.group(3)=='i686' \
                       or m.group(3)=='ia64' or m.group(3)=='noarch':

                    #what a strange idea to install a sysreport
                    #from rhel3 on a As2,1

                    v3forV2Flag=1
                    prhel3=re.compile(r"^(.*)-(.*)-(.*)-(.*)$")
                    mrhel3=prhel3.match(r)
                    if mrhel3:
                        curName=mrhel3.group(1)
                        curVersion=mrhel3.group(2)
                        curRelease=mrhel3.group(3)
                        if not self.options.arch:
                            curArch=mrhel3.group(4)
                        else:
                            curArch=self.options.arch
                    else :
                        globals.die("Problem parsing installed-rpms")
                    tmpDic={}
                    tmpDic['package_name']=curName
                    tmpDic['package_version']=curVersion
                    tmpDic['package_release']=curRelease
                    tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            else:
                globals.die("Cannot parse the following in installed-rpms: '%s'"%r.rstrip())
        RPMS.close()
        if v3forV2Flag:
            syslog.syslog(syslog.LOG_DEBUG, "Sysreport for rhel3 used on a AS2.1")
        return out

    def detect_SystemID(self):
        # added by Michael Roessler
        # read the systemid from config file
        out="unknown"
        systemidFile=self.sysreportPath+'/etc/sysconfig/rhn/systemid'
        candidates=globals.SimpleSet()

        try:
            f=open(systemidFile,'r')
            for line in f:
                line = line[:-1]
                split_line=line.split(":")
                if split_line[0].startswith("system_id"):
                        systemid = split_line[1][4:]
                        return systemid
        except IOError:
                return out


    def detect_baseChannel_RHN(self, systemid):
        # added by Michael Roessler
        # try to read basechannel from RHN
        basechannel = self.list_subscribed_base_channel(systemid)
        return basechannel


    def detect_baseChannel(self):
        base = "unknown"
        # added support for detection via RHN
        # Michael Roessler
        # we use system.list_base_channels to get the information
        systemid = self.detect_SystemID()
        if systemid and systemid != "unknown":
                        base=self.detect_baseChannel_RHN(systemid)
        if base == "unknown" or systemid == "unknown":
		"""
        	trying to detect base channel for AS 2.1 is less
        	easier than for RHEL3 or 4
        	Here are the available base channels:
        	redhat-advanced-server-i386
        	redhat-ent-linux-i386-es-2.1
        	redhat-ent-linux-i386-ws-2.1
        	redhat-linux-as-2.1-IPF
        	redhat-linux-aw-2.1-IPF
        	"""
        	arch=self.osArch
        	flavour=self.osFlavour
        	if arch=='i386':
            		if flavour=='as':
                		return 'redhat-advanced-server-i386'
            		if flavour=='es':
                		return 'redhat-ent-linux-i386-es-2.1'
            		if flavour=='ws':
                		return 'redhat-ent-linux-i386-ws-2.1'
                        globals.die("Unknown flavour for arch i386 on RHEL2: "+flavour)
        	if arch=='ia64':
            		if flavour=='as':
                		return 'redhat-linux-as-2.1-IPF'
            		#I wonder what this redhat-linux-aw-2.1-IPF channel may be.
            		#Lets wait untill i meet such a sysreport...
            		globals.die("Unknown flavour for arch i64 on RHEL2: "+flavour)

        	globals.die("Unable to detect base channel for RHEL2 with arch="+arch+" and flavour="+flavour)
                

        else:
                return base

    def list_known_amd_issues(self):
        out=[]
        if self.osArch=="x86_64" and \
           self.get_physical_address_size()>40:
            out.append("This cpu needs another version of RHEL:")
            out.append("http://kbase.redhat.com/faq/FAQ_85_11697")
        return out
        

    def list_known_kernel_issues(self):
        out=[]
        if self.is_amd():
            for p in self.list_known_amd_issues():
                out.append(p)
        return out


#####################################################################
#
#                            RHEL3 CLASS
#
#####################################################################

class sysreportChecker_3(sysreportChecker):
    """
    sysreportChecker for rhel3 based systems
    """

    def __init__(self,sysreportPath='',action=0,options=[]):
        self.osRelease='3'
        sysreportChecker.__init__(self,sysreportPath,options=options)
        
    def detect_osFlavour(self):
        redhatReleasePath=self.sysreportPath+'/etc/redhat-release'
        if os.path.isfile(redhatReleasePath):
            RELEASE=open(redhatReleasePath,'r')
            out='0'
            for line in RELEASE:
                reg=re.\
                     compile(r"Red Hat Enterprise Linux\s+(\w+)\s+release 3 .+")
                search=reg.search(line)
                deskReg=re.compile(r"Red Hat Desktop release 3 .*")

                if search:
                    out=search.group(1)
                    return out
                elif deskReg.search(line):
                    return "desktop"
                else:
                    globals.die("Cannot detect osFlavour for a rhel3")
        else:
            globals.die("Cannot find /etc/redhat-release")
            return out


    def detect_osArch(self):
        out="unknown"
        if self.options.arch:
            arch=self.options.arch
            if arch!="ia64" and arch!="i386" \
                   and arch != "x86_64" and arch != "ppc" \
                   and arch != "s390" and arch !="s390x":
                globals.die("Only the following arch are valid for RHEL3: i386,ia64,x86_64,ppc,s390")
            else:
                return arch

        
        installedRpmsFile=self.sysreportPath+'/installed-rpms'
        candidates=globals.SimpleSet()
        if os.path.isfile(installedRpmsFile):
            RPMS=open(installedRpmsFile,'r')
            for r in RPMS:
                p=re.compile(r".*-(\w+)$")
                m=p.search(r)
                if m:
                    candidates.add(m.group(1))
                    
            if candidates.is_member('x86_64'):
                return 'x86_64'
            elif candidates.is_member('ia64'):
                return 'ia64'
            elif candidates.is_member('ppc'):
                return 'ppc'
            elif candidates.is_member('s390x'):
                return 's390x'
            elif candidates.is_member('s390'):
                return 's390'
            elif candidates.is_member('i686'):
                return 'i386'
            elif candidates.is_member('i386'):
                return 'i386'
            else:
                globals.die("Unable to detect Arch: use -a option (ia64 i386 x86_64 ppc s390 s390x)")
                return out
                            
        else:
            globals.die("Cannot find /etc/installed-rpms")
            return out


    def detect_SystemID(self):
	# added by Michael Roessler
	# read the systemid from config file
	out="unknown"
        systemidFile=self.sysreportPath+'/etc/sysconfig/rhn/systemid'
        candidates=globals.SimpleSet()

        try:
            f=open(systemidFile,'r')
  	    for line in f:
            	line = line[:-1]
                split_line=line.split(":")
                if split_line[0].startswith("system_id"):
                       	systemid = split_line[1][4:]
			return systemid
	    f.close()
	    f=open(systemidFile,'r')
	    marker = 0
	    for line in f:
		if line.startswith("<name>system_id"):
			marker = 1
			continue
		if marker == 1:
			if line.startswith("<value><string>ID-"):
		       		splitline = line.split("-")
				systemid = splitline[1].split("<")[0]
				return systemid	
		      	
        except IOError:
		return out                
	

    def detect_baseChannel_RHN(self, systemid):
        # added by Michael Roessler
        # try to read basechannel from RHN	
	basechannel = self.list_subscribed_base_channel(systemid)
        return basechannel


    def detect_baseChannel(self):
        base = "unknown"
	# added support for detection via RHN 
	# Michael Roessler
	# we use system.list_base_channels to get the information
	systemid = self.detect_SystemID()
	if systemid and systemid != "unknown":
			base=self.detect_baseChannel_RHN(systemid)
  	if base == "unknown" or systemid == "unknown":	
            	base='rhel-'+self.osArch+'-'+self.osFlavour+'-'+self.osRelease
        	if base in  self.list_base_channels():
            		return base
        	else:
            		globals.die("Unable to detect base channel with hint="+base)
	else:
		return base


    def getInstalledRpms(self):
        out=[]
        RPMS=open(self.sysreportPath+"/installed-rpms",'r')
        for r in RPMS:
            if self.options.arch:
                p=re.compile(r"^(.*)-(.*)-(.*)$")
            else:
                p=re.compile(r"^(.*)-(.*)-(.*)-(.*)$")
            m=p.match(r)
            if m:
                curName=m.group(1)
                curVersion=m.group(2)
                curRelease=m.group(3)
                if not self.options.arch:
                    curArch=m.group(4)
                else:
                    curArch=self.options.arch
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            else:
                globals.die("Cannot parse '"+r.rstrip()+"' in installed-rpms. Try to use the -a option for forcing arch.")
        RPMS.close()
        return out

    def list_known_amd_issues(self):
        out=[]
        if self.osArch=="x86_64" and \
           self.get_physical_address_size()>40:
            out.append("This cpu needs another version of RHEL:")
            out.append("http://kbase.redhat.com/faq/FAQ_85_11697")
        return out
        
    def list_known_kernel_issues(self):
        out=[]
        if self.is_amd():
            for p in self.list_known_amd_issues():
                out.append(p)
        return out


#####################################################################
#
#                            RHEL4 CLASS
#
#####################################################################

class sysreportChecker_4(sysreportChecker):
    """
    sysreportChecker for rhel4 based systems
    """

    def __init__(self,sysreportPath='',action=0,options=[]):
        self.osRelease='4'
        sysreportChecker.__init__(self,sysreportPath,options=options)

    def detect_osFlavour(self):
        redhatReleasePath=self.sysreportPath+'/etc/redhat-release'
        if os.path.isfile(redhatReleasePath):
            RELEASE=open(redhatReleasePath,'r')
            out='0'
            for line in RELEASE:
                reg=re.\
                     compile(r"Red Hat Enterprise Linux\s+(\w+)\s+release 4 .+")
                search=reg.search(line)
                deskReg=re.\
                         compile(r"Red Hat Desktop release 4 .*")
                if search:
                    out=search.group(1)
                    RELEASE.close()
                    return out
                elif deskReg.search(line):
                    RELEASE.close()
                    return "desktop"
                else:
                    globals.die("Cannot detect osFlavour for a rhel4")
        else:
            globals.die("Cannot find /etc/redhat-release")

    def detect_osArch(self):
        out="unknown"
        installedRpmsFile=self.sysreportPath+'/installed-rpms'
        candidates=globals.SimpleSet()

        if self.options.arch:
            arch=self.options.arch
            if arch!="ia64" and arch!="i386" \
                   and arch != "x86_64" and arch != "ppc" \
                   and arch != "s390" and arch !="s390x":
                globals.die("Only the following arch are valid for RHEL4: i386,ia64,x86_64,ppc,s390")
            else:
                return arch



        if os.path.isfile(installedRpmsFile):
            RPMS=open(installedRpmsFile,'r')
            for r in RPMS:
                p=re.compile(r".*-(\w+)\s+.*$")
                m=p.search(r)
                if m:
                    candidates.add(m.group(1))
        
            if candidates.is_member('x86_64'):
                return 'x86_64'
            elif candidates.is_member('ia64'):
                return 'ia64'
            elif candidates.is_member('ppc'):
                return 'ppc'
            elif candidates.is_member('s390x'):
                return 's390x'
            elif candidates.is_member('s390'):
                return 's390'
            elif candidates.is_member('i686'):
                return 'i386'
            elif candidates.is_member('i386'):
                return 'i386'
            else:
                syslog.syslog(syslog.LOG_DEBUG,"Unable to detect Arch")
                return out
                            
        else:
            syslog.syslog(syslog.LOG_DEBUG,"Cannot find /etc/installed-rpms")
            return out

    def detect_SystemID(self):
	# added by Michael Roessler
	# read the systemid from config file
	out="unknown"
        systemidFile=self.sysreportPath+'/etc/sysconfig/rhn/systemid'
        candidates=globals.SimpleSet()

        try:
            f=open(systemidFile,'r')
  	    for line in f:
            	line = line[:-1]
                split_line=line.split(":")
                if split_line[0].startswith("system_id"):
                       	systemid = split_line[1][4:]
			return systemid
        except IOError:
		return out                
	

    def detect_baseChannel_RHN(self, systemid):
        # added by Michael Roessler
        # try to read basechannel from RHN	
	basechannel = self.list_subscribed_base_channel(systemid)
        return basechannel
        
    def detect_baseChannel(self):
        base = "unknown"
        """
         If the --s-stream option is used, let's override the base channel
         detection.
        """
        if self.options.zstream:
            zstream=self.options.zstream
            for cand in self.list_base_channels():
                if zstream==cand:
                    return cand
                
            """
            not existing z-stream.
            """
            message="No such z-stream (%s) for RHEL4, you should try out one of the following:\n"%zstream
            for i in self.guess_existing_z_streams():
                            message+=i+"\n"
            globals.die(message)

        
	# added support for detection via RHN 
	# Michael Roessler
	# we use system.list_base_channels to get the information
	systemid = self.detect_SystemID()
	if systemid and systemid != "unknown":
			base=self.detect_baseChannel_RHN(systemid)
  	if base == "unknown" or systemid == "unknown" or not systemid:	
        	if self.osFlavour=="desktop":
            		base='rh-'+self.osArch+'-'+self.osFlavour+'-'+self.osRelease
        	else:
            		base='rhel-'+self.osArch+'-'+self.osFlavour+'-'+self.osRelease

        	if base in  self.list_base_channels():
            		return base
        	else:
            		globals.die("Unable to detect base channel with hint="+base)
	else:
		return base

    def getInstalledRpms(self):
        out=[]
        RPMS=open(self.sysreportPath+"/installed-rpms",'r')
        for r in RPMS:
            p=re.compile(r"^(.*)-(.*)-(.*)-(\S+)\s+.*$")
            q=re.compile(r"^(.*)-(.*)-(.*?)$")
            m=p.match(r)
            n=q.match(r)
            if m and not self.options.arch:
                curName=m.group(1)
                curVersion=m.group(2)
                curRelease=m.group(3)
                curArch=m.group(4)
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            elif n:
                """
                Strange, some sysreport comes without arch information
                cf crm#911298
                """
                if not self.options.arch:
                    globals.die("Cannot detect arch for RHEL4, use the -a option")
                curName=n.group(1)
                curVersion=n.group(2)
                curRelease=n.group(3)
                curArch=self.options.arch
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            else:
                globals.die("Cannot parse the following in installed-rpms: '%s'"%r.rstrip())
        RPMS.close()
        return out


    def get_supported_kernel_modules(self):
        """
         On RHEL4's 2.6 , some kernel module appears loaded with another name
         than the actual module.ko :
         uhci_hcd -> uhci-hcd.ko
         snd_intel8x0 -> snd-intel8x0.ko
        """
        out=[]
        candidates=sysreportChecker.get_supported_kernel_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out
    
    def get_loaded_modules(self):
        """
         As in get_supported_kernel_modules, we s/-/_/ in all modules's name.
        """
        out=[]
        candidates=sysreportChecker.get_loaded_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out

    def list_known_amd_issues(self):
        """
        http://kbase.redhat.com/faq/FAQ_85_11697
        If running in 64 bit mode, and address sizes >40
        the kernel must be>kernel-2.6.9-55-0.12 (pid=422814)
        """
        out=[]
        if self.osArch=="x86_64" :
            addr_size=self.get_physical_address_size()
            running=self.getRunningKernel()
            min_kernel=globals.get_rpm_infos(422814,'rhel-x86_64-as-4')
            if globals.rpm_labelCompare(running,min_kernel)<=0 and addr_size>40:
                out.append("Upgrading kernel is mandatory to avoid crash:")
                out.append("http://kbase.redhat.com/faq/FAQ_85_11697")
        return out
        

    def list_known_kernel_issues(self):
        out=[]
        if self.is_amd():
            for p in self.list_known_amd_issues():
                out.append(p)
            

        return out


#####################################################################
#
#                            RHEL5 CLASS
#
#####################################################################

class sysreportChecker_5(sysreportChecker):
    """
    sysreportChecker for rhel5 based systems
    """

    def __init__(self,sysreportPath='',action=0,options=[]):
        self.osRelease='5'
        sysreportChecker.__init__(self,sysreportPath,options=options)


    def detect_osFlavour(self):
        redhatReleasePath=self.sysreportPath+'/etc/redhat-release'
        if os.path.isfile(redhatReleasePath):
            RELEASE=open(redhatReleasePath,'r')
            out='0'
            for line in RELEASE:
                reg=re.compile(r"Red Hat Enterprise Linux\s+(\S+)\s+release 5.*\(Tikanga\)")
                search=reg.search(line)
                if search:
                    RELEASE.close()
                    return search.group(1)
                else:
                    globals.die("Cannot detect osFlavour for a rhel5 with :\n%s" % line)
        else:
            globals.die("Cannot find /etc/redhat-release")
            
    def detect_osArch(self):
        out="unknown"
        installedRpmsFile=self.sysreportPath+'/installed-rpms'
        candidates=globals.SimpleSet()

        if self.options.arch:
            arch=self.options.arch
            if arch!="ia64" and arch!="i386" \
                   and arch != "x86_64" and arch != "ppc" \
                   and arch !="s390x":
                globals.die("Only the following arch are valid for RHEL5: i386,ia64,x86_64,ppc,s390x")
            else:
                return arch


        if os.path.isfile(installedRpmsFile):
            RPMS=open(installedRpmsFile,'r')
            for r in RPMS:
                p=re.compile(r".*[\.-](\w+)\s+.*$")
                m=p.search(r)
                if m:
                    candidates.add(m.group(1))

            if candidates.is_member('x86_64'):
                return 'x86_64'
            elif candidates.is_member('ia64'):
                return 'ia64'
            elif candidates.is_member('ppc'):
                return 'ppc'
            elif candidates.is_member('s390x'):
                return 's390x'
            elif candidates.is_member('i686'):
                return 'i386'
            elif candidates.is_member('i386'):
                return 'i386'
            else:
                syslog.syslog(syslog.LOG_DEBUG, "Unable to detect Arch")
                return out
                            
        else:
            syslog.syslog(syslog.LOG_DEBUG,"Cannot find /etc/installed-rpms")
            return out


    def getInstalledRpms(self):
        out=[]
        RPMS=open(self.sysreportPath+"/installed-rpms",'r')
        #Grmblblblbbl BZ#482755 is a pain here....
        #sos-1.7-9.49.el5.noarch                                     Thu 01 Apr 2010 01:36:13 PM CEST
        #
        for r in RPMS:
            p=re.compile(r"^(.*)-(.*)-(.*)[\.-](\S+)\s+.*$")
            q=re.compile(r"^(.*)-(.*)-(.*?)$")
            m=p.match(r)
            n=q.match(r)

            if m and not self.options.arch:
                curName=m.group(1)
                curVersion=m.group(2)
                curRelease=m.group(3)
                curArch=m.group(4)
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            elif n:
                """
                Strange, some sysreport comes without arch information
                cf crm#911298
                """
                if not self.options.arch:
                    globals.die("Cannot detect arch for RHEL5, use the -a option")
                curName=n.group(1)
                curVersion=n.group(2)
                curRelease=n.group(3)
                curArch=self.options.arch
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            else:
                globals.die("Cannot parse the following in installed-rpms :'%s'"%r.rstrip())
        RPMS.close()
        return out

    def detect_SystemID(self):
        # added by Michael Roessler
        # read the systemid from config file
        out="unknown"
        systemidFile=self.sysreportPath+'/etc/sysconfig/rhn/systemid'
        candidates=globals.SimpleSet()

        try:
            f=open(systemidFile,'r')
            for line in f:
                line = line[:-1]
                split_line=line.split(":")
                if split_line[0].startswith("system_id"):
                        systemid = split_line[1][4:]
                        return systemid
        except IOError:
                return out


    def detect_baseChannel_RHN(self, systemid):
        # added by Michael Roessler
        # try to read basechannel from RHN
        basechannel = self.list_subscribed_base_channel(systemid)
        return basechannel


    def detect_baseChannel(self):
        base = "unknown"
        """
        If the --s-stream option is used, let's override the base channel
        detection.
        """
        if self.options.zstream:
            zstream=self.options.zstream
            for cand in self.list_base_channels():
                if zstream==cand:
                    return cand
                
            """
            not existing z-stream.
            """
            message= "No such z-stream (%s) for RHEL5, you should try out one of the following:\n"%zstream
            for i in self.guess_existing_z_streams():
                message+=i+"\n"
            globals.die(message)

        # added support for detection via RHN
        # Michael Roessler
        # we use system.list_base_channels to get the information
        systemid = self.detect_SystemID()
        if systemid and systemid != "unknown":
                        base=self.detect_baseChannel_RHN(systemid)
        if base == "unknown" or systemid == "unknown":
                base='rhel-'+self.osArch+'-'+self.osFlavour+'-5'
                if base in  self.list_base_channels():
                        return base
                else:
                        globals.die("Unable to detect base channel with hint="+base)
        else:
                return base




    def get_supported_kernel_modules(self):
        """
        Cf RHEL4
        """
        out=[]
        candidates=sysreportChecker.get_supported_kernel_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out
    
    def get_loaded_modules(self):
        """
         As in get_supported_kernel_modules, we s/-/_/ in all modules's name.
        """
        out=[]
        candidates=sysreportChecker.get_loaded_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out
            

    def list_known_amd_issues(self):
        """
        None yet
        """
        out=[]
        return out
        

    def list_known_kernel_issues(self):
        out=[]
        if self.is_amd():
            for p in self.list_known_amd_issues():
                out.append(p)
        return out


#####################################################################
#
#                            RHEL6 CLASS
#
#####################################################################

class sysreportChecker_6(sysreportChecker):
    """
    sysreportChecker for rhel6 based systems
    """

    def __init__(self,sysreportPath='',action=0,options=[]):
        self.osRelease='6'
        sysreportChecker.__init__(self,sysreportPath,options=options)

    def detect_osFlavour(self):
        """
        Red Hat Enterprise Linux Server release 6.0 (Santiago)
        Red Hat Enterprise Linux Client release 6.0 (Santiago)
        Red Hat Enterprise Linux Workstation release 6.0 (Santiago)
        """
        redhatReleasePath=self.sysreportPath+'/etc/redhat-release'
        if os.path.isfile(redhatReleasePath):
            RELEASE=open(redhatReleasePath,'r')
            out='0'
            for line in RELEASE:
                reg=re.compile(r"Red Hat Enterprise Linux\s+(\S+)\s+release 6.* \(Santiago\)")
                search=reg.search(line)
                if search:
                    RELEASE.close()
                    return search.group(1)
                else:
                    globals.die("Cannot detect osFlavour for a rhel6 with :\n%s" % line)
        else:
            globals.die("Cannot find /etc/redhat-release")

    def detect_osArch(self):
        out="unkown"
        installedRpmsFile=self.sysreportPath+"/installed-rpms"
        candidates=globals.SimpleSet()

        if self.options.arch:
            arch=self.options.arch
            if arch!="ia64" and arch!="i386" \
                   and arch != "x86_64" and arch != "ppc" \
                   and arch !="s390x":
                globals.die("Only the following arch are valid for RHEL6: i386,ia64,x86_64,ppc64,s390x")
            else:
                return arch

        if os.path.isfile(installedRpmsFile):
            RPMS=open(installedRpmsFile,'r')
            for r in RPMS:
                p=re.compile(r".*[\.-](\w+)\s+.*$")
                m=p.search(r)
                if m:
                    candidates.add(m.group(1))

            if candidates.is_member('x86_64'):
                return 'x86_64'
            elif candidates.is_member('ia64'):
                return 'ia64'
            elif candidates.is_member('ppc64'):
                return 'ppc64'
            elif candidates.is_member('s390x'):
                return 's390x'
            elif candidates.is_member('i686'):
                return 'i386'
            elif candidates.is_member('i386'):
                return 'i386'
            else:
                syslog.syslog(syslog.LOG_DEBUG, "Unable to detect Arch")
                return out
                            
        else:
            syslog.syslog(syslog.LOG_DEBUG,"Cannot find /etc/installed-rpms")
            return out


    def getInstalledRpms(self):
        out=[]
        RPMS=open(self.sysreportPath+"/installed-rpms",'r')
        for r in RPMS:
            p=re.compile(r"^(.*)-(.*)-(.*)[\.-](\S+)\s+.*$")
            m=p.match(r)
            if m:
                curName=m.group(1)
                curVersion=m.group(2)
                curRelease=m.group(3)
                curArch=m.group(4)
                tmpDic={}
                tmpDic['package_name']=curName
                tmpDic['package_version']=curVersion
                tmpDic['package_release']=curRelease
                tmpDic['package_arch_label']=curArch
                out.append(tmpDic)
            else:
                globals.die("Canot parse the following in installed-rpms: '%s'"%r.rstrip())
        RPMS.close()
        return out
                            
    def detect_SystemID(self):
        # added by Michael Roessler
        # read the systemid from config file
        out="unknown"
        systemidFile=self.sysreportPath+'/etc/sysconfig/rhn/systemid'
        candidates=globals.SimpleSet()

        try:
            f=open(systemidFile,'r')
            for line in f:
                line = line[:-1]
                split_line=line.split(":")
                if split_line[0].startswith("system_id"):
                        systemid = split_line[1][4:]
                        return systemid
        except IOError:
                return out

    def detect_baseChannel_RHN(self, systemid):
        # added by Michael Roessler
        # try to read basechannel from RHN
        basechannel = self.list_subscribed_base_channel(systemid)
        return basechannel

    def detect_baseChannel(self):
        base = "unknown"
        """
        If the --s-stream option is used, let's override the base channel
        detection.
        """
        if self.options.zstream:
            zstream=self.options.zstream
            for cand in self.list_base_channels():
                if zstream==cand:
                    return cand
                
            """
            not existing z-stream.
            """
            message= "No such z-stream (%s) for RHEL6, you should try out one of the following:\n"%zstream
            for i in self.guess_existing_z_streams():
                message+=i+"\n"
            globals.die(message)

        # added support for detection via RHN
        # Michael Roessler
        # we use system.list_base_channels to get the information
        systemid = self.detect_SystemID()
        if systemid and systemid != "unknown":
                        base=self.detect_baseChannel_RHN(systemid)
        if base == "unknown" or systemid == "unknown":
                base='rhel-'+self.osArch+'-'+self.osFlavour+'-6'
                if base in  self.list_base_channels():
                        return base
                else:
                        globals.die("Unable to detect base channel with hint="+base)
        else:
                return base


    def get_supported_kernel_modules(self):
        """
        Cf RHEL4
        """
        out=[]
        candidates=sysreportChecker.get_supported_kernel_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out
    
    def get_loaded_modules(self):
        """
         As in get_supported_kernel_modules, we s/-/_/ in all modules's name.
        """
        out=[]
        candidates=sysreportChecker.get_loaded_modules(self)
        for c in candidates:
            new=string.replace(c,'-','_')
            out.append(new)
        return out


    def get_kernel_version(self):
       """
        Return the kernel version used when the sysreport was launched.
        such as : 2.6.9-5.0.5.EL  2.4.21-27.0.4.ELsmp and so on.

        This is overloaded for RHEL6 because the uname file seems to
        add arch information that was not there previously:
        Linux localhost.localdomain 2.6.32-71.el6.x86_64 #1 SMP Wed Sep 1 01:33:01 EDT 2010 x86_64 x86_64 x86_64 GNU/Linux
        
        
       """
       uname=self.get_file('uname')
       reg=re.compile(r"^Linux (\S+) (\S+)\.(\S+) #",re.MULTILINE)
       if reg.search(uname):
           return reg.search(uname).group(2)
       else:
           globals.die("Cannot detect kernel version with:"+uname)
       


            

    def list_known_amd_issues(self):
        """
        None yet
        """
        out=[]
        return out
        

    def list_known_kernel_issues(self):
        out=[]
        if self.is_amd():
            for p in self.list_known_amd_issues():
                out.append(p)
        return out



